"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3Helper = void 0;
const S3 = require("@aws-sdk/client-s3");
const s3_request_presigner_1 = require("@aws-sdk/s3-request-presigner");
const typescript_helper_functions_1 = require("typescript-helper-functions");
const stream_1 = require("stream");
class S3Helper extends typescript_helper_functions_1.BaseClass {
    Repository;
    Options;
    constructor(logger, repository, options) {
        super(logger);
        this.Options = this.ObjectOperations.IsNullOrEmpty(options)
            ? { region: 'us-east-1' }
            :
                options;
        this.Repository = repository || new S3.S3(this.Options);
    }
    async CopyObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey) {
        const action = `${S3Helper.name}.${this.CopyObjectAsync.name}`;
        this.LogHelper.LogInputs(action, {
            sourceBucket,
            sourceKey,
            destinationBucket,
            destinationKey,
        });
        if (this.ObjectOperations.IsNullOrWhitespace(sourceBucket)) {
            throw new Error(`[${action}]-Must supply sourceBucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(sourceKey)) {
            throw new Error(`[${action}]-Must supply sourceKey`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(destinationBucket)) {
            throw new Error(`[${action}]-Must supply destinationBucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(destinationKey)) {
            throw new Error(`[${action}]-Must supply destinationKey`);
        }
        const params = {
            Bucket: destinationBucket,
            CopySource: `${sourceBucket}/${sourceKey}`,
            Key: destinationKey,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.copyObject(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async CreateBucketAsync(name) {
        const action = `${S3Helper.name}.${this.CreateBucketAsync.name}`;
        this.LogHelper.LogInputs(action, { name });
        if (this.ObjectOperations.IsNullOrWhitespace(name)) {
            throw new Error(`[${action}]-Must supply name`);
        }
        const params = {
            Bucket: name,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.createBucket(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async DeleteBucketAsync(name) {
        const action = `${S3Helper.name}.${this.DeleteBucketAsync.name}`;
        this.LogHelper.LogInputs(action, { name });
        if (this.ObjectOperations.IsNullOrWhitespace(name)) {
            throw new Error(`[${action}]-Must supply name`);
        }
        const params = {
            Bucket: name,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.deleteBucket(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async DeleteObjectAsync(bucket, key) {
        const action = `${S3Helper.name}.${this.DeleteObjectAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.deleteObject(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async DeleteObjectsAsync(bucket, keys) {
        const action = `${S3Helper.name}.${this.DeleteObjectsAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, keys });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (!keys || keys.length === 0) {
            throw new Error(`[${action}]-Must supply at least one key`);
        }
        const keysArray = [];
        for (const key of keys) {
            keysArray.push({ Key: key });
        }
        const params = {
            Bucket: bucket,
            Delete: { Objects: keysArray },
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.deleteObjects(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async DeleteObjectTagsAsync(bucket, key) {
        const action = `${S3Helper.name}.${this.DeleteObjectTagsAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.deleteObjectTagging(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async GetBucketMetadataAsync(bucket) {
        const action = `${S3Helper.name}.${this.GetBucketMetadataAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        const params = {
            Bucket: bucket,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.headBucket(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async GetObjectAsJsonAsync(bucket, key) {
        const data = await this.GetObjectContentsAsync(bucket, key);
        const json = data ? data.toString() : '';
        return JSON.parse(json);
    }
    async GetObjectAsync(bucket, key) {
        const action = `${S3Helper.name}.${this.GetObjectAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.getObject(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async GetObjectContentsAsync(bucket, key) {
        const data = (await this.GetObjectAsync(bucket, key)).Body;
        return data ? data : undefined;
    }
    async GetObjectMetadataAsync(bucket, key) {
        const action = `${S3Helper.name}.${this.GetObjectMetadataAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.headObject(params);
        this.LogHelper.LogResponse(action, response);
        return response.Metadata;
    }
    async GetObjectTagsAsync(bucket, key) {
        const action = `${S3Helper.name}.${this.GetObjectTagsAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.getObjectTagging(params);
        this.LogHelper.LogResponse(action, response);
        return response.TagSet ?? [];
    }
    async GetSignedUrlDownload(bucket, key, timeoutInMinutes = 5) {
        const action = `${S3Helper.name}.${this.GetSignedUrlDownload.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, timeoutInMinutes });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
        };
        this.LogHelper.LogRequest(action, params);
        const client = new S3.S3Client(this.Options);
        const operation = new S3.GetObjectCommand(params);
        const response = await (0, s3_request_presigner_1.getSignedUrl)(client, operation, {
            expiresIn: timeoutInMinutes * 60,
        });
        this.LogHelper.LogResponse(action, { response });
        return response;
    }
    async GetSignedUrlUpload(bucket, key, timeoutInMinutes = 5, acl) {
        const action = `${S3Helper.name}.${this.GetSignedUrlUpload.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, acl, timeoutInMinutes });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
            ACL: acl,
        };
        this.LogHelper.LogRequest(action, params);
        const client = new S3.S3Client(this.Options);
        const operation = new S3.PutObjectCommand(params);
        const response = await (0, s3_request_presigner_1.getSignedUrl)(client, operation, {
            expiresIn: timeoutInMinutes * 60,
        });
        this.LogHelper.LogResponse(action, { response });
        return response;
    }
    async MoveObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey) {
        const copyResult = await this.CopyObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey);
        if (!this.ObjectOperations.IsNullOrEmpty(copyResult)) {
            await this.DeleteObjectAsync(sourceBucket, sourceKey);
        }
    }
    async MultipartUploadCompleteAsync(bucket, key, uploadId) {
        const action = `${S3Helper.name}.${this.MultipartUploadCompleteAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, uploadId });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(uploadId)) {
            throw new Error(`[${action}]-Must supply uploadId`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
            UploadId: uploadId,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.completeMultipartUpload(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async MultipartUploadStartAsync(bucket, key, acl) {
        const action = `${S3Helper.name}.${this.MultipartUploadStartAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, acl });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
            ACL: acl,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.createMultipartUpload(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async MultipartUploadUploadPartAsync(bucket, key, uploadId, uploadPart, contents) {
        const action = `${S3Helper.name}.${this.MultipartUploadUploadPartAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, uploadId, uploadPart });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(uploadId)) {
            throw new Error(`[${action}]-Must supply uploadId`);
        }
        if (uploadPart === 0) {
            throw new Error(`[${action}]-Cannot be zero - uploadId`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(contents)) {
            throw new Error(`[${action}]-Must supply contents`);
        }
        const bytes = new TextEncoder().encode(contents);
        const tooSmall = bytes.length < 5000000;
        const tooLarge = bytes.length > 10000000000;
        if (tooSmall || tooLarge) {
            throw new Error(`[${action}]-Part size must be between 5 MB and 10 GB`);
        }
        const readable = new stream_1.Readable();
        readable.push(contents);
        readable.push(null);
        const params = {
            Body: readable,
            Bucket: bucket,
            Key: key,
            PartNumber: uploadPart,
            UploadId: uploadId,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.uploadPart(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async PutObjectAsync(bucket, key, contents, acl, encoding) {
        const action = `${S3Helper.name}.${this.PutObjectAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, acl });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(acl)) {
            acl = 'bucket-owner-full-control';
        }
        if (this.ObjectOperations.IsNullOrWhitespace(encoding)) {
            encoding = 'utf-8';
        }
        let body = new stream_1.Readable();
        switch (typeof contents) {
            case 'undefined':
                throw new Error(`[${action}]-Must supply contents`);
            case 'string':
                body.push(contents);
                body.push(null);
                break;
            default:
                body = contents;
                break;
        }
        const params = {
            ACL: acl,
            Body: body,
            Bucket: bucket,
            ContentEncoding: encoding,
            Key: key,
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.putObject(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
    async SetObjectTagAsync(bucket, key, tagName, tagValue) {
        const action = `${S3Helper.name}.${this.SetObjectTagAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, tagName, tagValue });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(tagName)) {
            throw new Error(`[${action}]-Must supply tagName`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(tagValue)) {
            throw new Error(`[${action}]-Must supply tagValue`);
        }
        const tags = await this.GetObjectTagsAsync(bucket, key);
        const tag = tags.find((t) => t.Key === tagName);
        if (tag) {
            tag.Value = tagValue;
        }
        else {
            tags.push({ Key: tagName, Value: tagValue });
        }
        return this.SetObjectTagsAsync(bucket, key, tags);
    }
    async SetObjectTagsAsync(bucket, key, tags) {
        const action = `${S3Helper.name}.${this.SetObjectTagsAsync.name}`;
        this.LogHelper.LogInputs(action, { bucket, key, tags });
        if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
            throw new Error(`[${action}]-Must supply bucket`);
        }
        if (this.ObjectOperations.IsNullOrWhitespace(key)) {
            throw new Error(`[${action}]-Must supply key`);
        }
        if (this.ObjectOperations.IsNullOrEmpty(tags)) {
            throw new Error(`[${action}]-Must supply tags`);
        }
        const params = {
            Bucket: bucket,
            Key: key,
            Tagging: { TagSet: tags },
        };
        this.LogHelper.LogRequest(action, params);
        const response = await this.Repository.putObjectTagging(params);
        this.LogHelper.LogResponse(action, response);
        return response;
    }
}
exports.S3Helper = S3Helper;
//# sourceMappingURL=helper.js.map